#  MultiUtility
#  Copyright (C) 2005-2021
#  This file is part of MultiUtility.
#
#  MultiUtility is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MultiUtility is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MultiUtility.  If not, see <http://www.gnu.org/licenses/>.

import requests
import aiohttp
import asyncio
from urllib.request import urlretrieve

class MakeRequest:
  def __init__(self):
    """Sync Connection"""
  
  def get(self, url, **kwargs):
    """Get Content"""
    return requests.get(url, **kwargs)
  
  def post(self, url, **kwargs):
    """POST Content"""
    return requests.set(url, **kwargs)
  
  def put(self, url, **kwargs):
    """PUT Content"""
    return requests.put(url, **kwargs)
  
  def dl(self, url, file, **kwargs):
    """Download a File."""
    urlretrieve(url, file, **kwargs)
    return file
  
  @property
  def open(self):
    """Deprecated! for backward compatiblity"""
    return True


class AsyncRequest:
  def __init__(self):
    """Async Connection"""
    self.client = aiohttp
  
  async def get(self, url):
    """GET Content."""
    async with self.client.ClientSession() as session:
      async with session.get(url) as content:
        return await content.read()

  async def post(self, url):
    """POST Content."""
    async with self.client.ClientSession() as session:
      async with session.post(url) as content:
        return await content.read()

  async def put(self, url):
    # check issues #153, #121
    # TODO: Rewrite
    pass
  
  async def dl(self, url, name):
    """Download file."""
    async with self.client.ClientSession() as session:
      async with session.get(url) as content:
        open(name, "wb").write(await content.read())
        return name


